#pragma once

#ifndef SYNCHRONIZED_TEXTURE_HEAP_HOST_H
#define SYNCHRONIZED_TEXTURE_HEAP_HOST_H




#include "synchronized_texture_heap_base.h"
#include "platform.h"
#include "color.h"

#include <boost/shared_ptr.hpp>
#include <boost/shared_array.hpp>

#include <array>




namespace Synchronized {




/*==============================================================================
	TextureHeap< PLATFORM_HOST > specialization
==============================================================================*/


template< int t_ColorFormat >
struct TextureHeap< t_ColorFormat, PLATFORM_HOST > {

	struct Pointer;


	inline TextureHeap();


	inline Pointer Allocate( unsigned int const logSize ) const;


	inline void Synchronize() const;


	struct Pointer : public boost::shared_array< Color< t_ColorFormat > const > {    // **TODO: in boost 1.53+, shared_ptr< Color< t_ColorFormat >[] > can substitute for shared_array< Color< t_ColorFormat > >

		typedef typename boost::shared_ptr< Color< t_ColorFormat > const >::element_type element_type;


		inline Pointer();
		inline Pointer( Pointer const& other );


		inline void Assign( size_t const index, element_type const& data ) const;


	private:

		inline Pointer( Color< t_ColorFormat >* array );


		friend struct TextureHeap;
	};
};




/*==============================================================================
	TextureHeap< PLATFORM_HOST >::Pointer methods
==============================================================================*/


template< int t_ColorFormat >
TextureHeap< t_ColorFormat, PLATFORM_HOST >::Pointer::Pointer() {
}


template< int t_ColorFormat >
TextureHeap< t_ColorFormat, PLATFORM_HOST >::Pointer::Pointer( Pointer const& other ) : boost::shared_array< Color< t_ColorFormat > const >( other ) {
}


template< int t_ColorFormat >
void TextureHeap< t_ColorFormat, PLATFORM_HOST >::Pointer::Assign( size_t const index, element_type const& data ) const {

	*const_cast< Color< t_ColorFormat >* >( boost::shared_array< Color< t_ColorFormat > const >::get() + index ) = data;
}


template< int t_ColorFormat >
TextureHeap< t_ColorFormat, PLATFORM_HOST >::Pointer::Pointer( Color< t_ColorFormat >* array ) : boost::shared_array< Color< t_ColorFormat > const >( array ) {
}




/*==============================================================================
	TextureHeap< PLATFORM_HOST > methods
==============================================================================*/


template< int t_ColorFormat >
TextureHeap< t_ColorFormat, PLATFORM_HOST >::TextureHeap() {
}


template< int t_ColorFormat >
typename TextureHeap< t_ColorFormat, PLATFORM_HOST >::Pointer TextureHeap< t_ColorFormat, PLATFORM_HOST >::Allocate( unsigned int const logSize ) const {

	return Pointer( new Color< t_ColorFormat >[ 1 << ( 3 * logSize ) ] );
}


template< int t_ColorFormat >
void TextureHeap< t_ColorFormat, PLATFORM_HOST >::Synchronize() const {
}




}    // namespace Synchronized




#endif    // SYNCHRONIZED_TEXTURE_HEAP_HOST_H
